#!/usr/bin/perl

$margin = 500;
$max_spanning_size = 100000000;

if(@ARGV < 3) {
 
    print ("\nUSAGE: fill_gaps.pl <scaff> <seqfile> <outdir>\n\n".
           "scaff\t= File containing scaffold information in .scaff format\n".
           "seqfile\t= Multi-Fasta file containing contig sequences\n".
           "outdir\t= Directory to which the files \"result.scaff\" and \"log\" will be written\n\n");
 
    exit;
}

($scaff, $sequences, $outdir, $fast) = @ARGV;

open(SCAFF, $scaff);
$data = join("", <SCAFF>);
close(SCAFF);

print "Step 1: Matching sequences to each other using Nucmer.\n";

#`nucmer --maxmatch $sequences $sequences -p $outdir/seq-v-seq 2> $outdir/log`;
#`show-coords -HTcl $outdir/seq-v-seq.delta > $outdir/seq-v-seq.coords`;
$match_results = "$outdir/seq-v-seq.coords";

print "Step 2: Testing to see if these matches fill any gaps.\n";
 
open(OUT, ">$outdir/result.scaff"); 
open(LOG, ">$outdir/log");

$count_good = 0;

#
# This code can work with multiple scaffolds!
#
while($data =~ m/(\>.*\n)([^\>]+)/g) {

    print OUT $1;
    @contigs = split(/\n/, $2);

    for $i (1..$#contigs) {

	($name1, $or1, $size1, $offset1) = split(/\s+/, $contigs[$i-1]);
	($name2, $or2, $size2, $offset2) = split(/\s+/, $contigs[$i]);

	$good = 0;

	$ctg_start1 = 1; $ctg_end1 = $size1;
	if($name1 =~ m/\{/) {
	    $name1 =~ s/(.*)\{(.*),(.*)\}/\1/; $ctg_start1 = $2; $ctg_end1 = $3;
	}

	$ctg_start2 = 1; $ctg_end2 = $size2;
	if($name2 =~ m/\{/) {
	    $name2 =~ s/(.*)\{(.*),(.*)\}/\1/; $ctg_start2 = $2; $ctg_end2 = $3;
	}

	$matches = `grep -P "$name1.*$name2" $match_results`;
	@matches = split(/\n/, $matches);

	print LOG "Analyzing gap: $name1 $or1 $name2 $or2 $offset1\n";
	printf OUT "$name1\{$ctg_start1," if($or1 eq "BE" && $i == 1);
	printf OUT "$name1\{$ctg_end1," if($or1 eq "EB" && $i == 1);

	for $j (0..$#matches) {

	    last if ($offset1 == 0);

	    ($start1, $finish1, $start2, $finish2, @rest) = split(/\s+/, $matches[$j]);

	    next if($name1 eq $name2 && $start1 eq $start2 && $finish1 eq $finish2);

	    # Fixing incorrect size entries
	    $ctg_end1 = $rest[3] if($size1 == $ctg_end1); $size1 = $rest[3]; 
	    $ctg_end2 = $rest[4] if($size2 == $ctg_end2); $size2 = $rest[4]; 

	    $good = 1
		if(($or1 eq "BE" && $finish1 >= $ctg_end1-$margin && $or2 eq "BE" && $start2 <= $ctg_start2+$margin) ||  
		   ($or1 eq "BE" && $finish1 >= $ctg_end1-$margin && $or2 eq "EB" && $start2 >= $ctg_end2-$margin) ||  
		   ($or1 eq "EB" && $start1 <= $ctg_start1+$margin && $or2 eq "BE" && $finish2 <= $ctg_start2+$margin) ||  
		   ($or1 eq "EB" && $start1 <= $ctg_start1+$margin && $or2 eq "EB" && $finish2 >= $ctg_end2-$margin));  

	    $count_good += $good;

	    if($good) {
	    
		$response = $fast;
		if(!$fast) {

		    print "\nLooking at Gap: $name1 $or1 $name2 $or2 $offset\n";
		    print "Contig ends overlap! Nucmer Match: \n$matches[$j]\nGood Enough? (Y/N): ";
		    $response = 1 if(($in = <STDIN>) =~ m/Y/);
		}
		
		if($response) {

		    print LOG "Contig ends overlap! Nucmer Match: \n$matches[$j]\n"; 
		    printf OUT "%d\} $or1 $size1 0\n", ($start1-1) if($or1 eq "BE");
		    printf OUT "%d\} $or1 $size1 0\n", ($finish1+1) if($or1 eq "EB");
		    printf OUT "$name2\{$start2," if($or1 eq "BE" && $i != $#contigs);
		    printf OUT "$name2\{$finish2," if($or1 eq "EB" && $i != $#contigs);
		}
	    }

	    last if($good);
	}

	if($good == 0) {

	    $matches = `grep -P "$name1.+" $match_results`;
	    @matches = split(/\n/, $matches);

	    OUTER: for $j (0..$#matches) {

		last if ($offset1 == 0);

		($start1, $finish1, $start2, $finish2, @rest) = split(/\s+/, $matches[$j]);
		$match = $rest[$#rest];
		$match_size = $rest[$#rest-4];

		next if($match eq $name1 || $match_size > $max_spanning_size);

		# Fixing incorrect size entries
		$ctg_end1 = $rest[3] if($size1 == $ctg_end1); $size1 = $rest[3]; 

		if((($or1 eq "BE" && $finish1 >= $ctg_end1-$margin) && (($start2 <= 1+$margin) || ($start2 >= $match_size-$margin))) ||  
		   (($or1 eq "EB" && $start1 <= $ctg_start1+$margin) && (($finish2 <= 1+$margin) || ($finish2 >= $match_size-$margin)))) {  

		    $match_or = (($or1 eq "BE" && $start2 <= 1+$margin && $start2 <= $finish2) ||
				 ($or1 eq "EB" && $finish2 <= 1+$margin && $start2 > $finish2) ? "BE" : "EB");

		    $other_matches = `grep -P "$match.+" $match_results`;
		    @other_matches = split(/\n/, $other_matches);

		    for $k (0..$#other_matches) {

			($o_start1, $o_finish1, $o_start2, $o_finish2, @o_rest) = split(/\s+/, $other_matches[$k]);

			$ctg_end2 = $o_rest[4] if($size2 == $ctg_end2); $size2 = $o_rest[4]; 
			
			$good = 1
			    if($o_rest[$#o_rest] eq $name2 &&
			       (($match_or eq "BE" && $o_finish1 >= $match_size-$margin && $or2 eq "BE" && $o_start2 <= $ctg_start2+$margin) ||  
				($match_or eq "BE" && $o_finish1 >= $match_size-$margin && $or2 eq "EB" && $o_start2 >= $ctg_end2-$margin) ||  
				($match_or eq "EB" && $o_start1 <= 1+$margin && $or2 eq "BE" && $o_finish2 <= $ctg_start2+$margin) ||  
				($match_or eq "EB" && $o_start1 <= 1+$margin && $or2 eq "EB" && $o_finish2 >= $ctg_end2-$margin)));  
			
			$count_good += $good;

			if($good) {

			    $response = $fast;
			    if(!$fast) {

				print "\nLooking at Gap: $name1 $or1 $name2 $or2 $offset\n";		    
				print "Found sequence $match that seems to span gap! Nucmer evidence: \n$matches[$j]\n$other_matches[$k]\nGood Enough? (Y/N): ";
				$response = 1 if(($in = <STDIN>) =~ m/Y/);
			    }
		
			    if($response) {

				print LOG "Found sequence $match that seems to span gap! Nucmer evidence: \n$matches[$j]\n$other_matches[$k]\n";			    
				printf OUT "%d\} $or1 $size1 0\n", ($start1-1) if($or1 eq "BE");
				printf OUT "%d\} $or1 $size1 0\n", ($finish1+1) if($or1 eq "EB");
				printf OUT "$match\{$start2," if($or1 eq "BE");
				printf OUT "$match\{$finish2," if($or1 eq "EB");

				printf OUT "%d\} $match_or $match_size 0\n", ($o_start1-1) if($match_or eq "BE");
				printf OUT "%d\} $match_or $match_size 0\n", ($o_finish1+1) if($match_or eq "EB");			    
				printf OUT "$name2\{$o_start2," if($match_or eq "BE" && $i != $#contigs);
				printf OUT "$name2\{$o_finish2," if($match_or eq "EB" && $i != $#contigs);
			    }
			}

			last OUTER if($good);
		    }
		}
	    }
	}

	if(!$good || ($good && !$response)) {

	    printf OUT "%d\} $or1 $size1 $offset1\n", ($or1 eq "BE" ? $ctg_end1 : $ctg_start1); 
	    printf OUT "$name2\{%d,", ($or2 eq "BE" ? $ctg_start2 : $ctg_end2);
	}

	print LOG ('-'x50 . "\n");
    }
    
    printf OUT "%d\} $or2 $size2 $offset2\n", ($or2 eq "BE" ? $ctg_end2 : $ctg_start2);
}

print "Successfully closed $count_good gaps!\n" if($fast);

close(OUT);
close(LOG);
