#!/usr/bin/perl

if(@ARGV < 2) {
 
    print ("\nUSAGE: suggest_contigs.pl <scaff> <seqfile>\n\n".
           "scaff\t= File containing scaffold information in .scaff format\n".
           "seqfile\t= Multi-Fasta file containing contigs and finishing sequences\n\n".
	   "The program will suggest contigs that are likely to be in large gaps in the scaffold\n".
           "based on the agreement of GC content of the ends.\n\n");
 
    exit;
}

($scaff, $sequences, $gc_diff, $large_gap_size) = @ARGV;

$gc_diff = 0.02 if($gc_diff == 0);
$large_gap_size = 5000 if($large_gap_size == 0);

print "Step 1: Computing GC content of 1Kbp ends.\n";

open(IN, $sequences);
$data = join("", <IN>);
while($data =~ m/\>(\S+).*\n([^\>]+)/g) {

    $name = $1; $seq = $2; $seq =~ s/\s//g; $length{$name} = length($seq); 
    
    if(length($seq) > 1000) {

	$begin = substr($seq, 0, 1000); $end = substr($seq, length($seq)-1000);
	$begin =~ s/[^GCgc]//g; $end =~ s/[^GCgc]//g; 
	$begin{$name} = length($begin)/1000; $end{$name} = length($end)/1000;
    }
}
close(IN);

print "Step 2: Testing to see if any of these fill the gaps.\n";

open(SCAFF, $scaff);
$data = join("", <SCAFF>);
close(SCAFF);

map { $placed{$_} = 1; } ($data =~ m/\n([^\>]\S+) /g);

foreach $contig (keys(%length)) {

    push @candidates, $contig if(!$placed{$contig} && $length{$contig} > $large_gap_size);
} 

print "Candidates: @candidates\n\n";

#
# This code can work with multiple scaffolds!
#
while($data =~ m/(\>.*\n)([^\>]+)/g) {

    @contigs = split(/\n/, $2);

    for $i (1..$#contigs) {

	($name1, $or1, $size1, $offset1) = split(/\s+/, $contigs[$i-1]);
	($name2, $or2, $size2, $offset2) = split(/\s+/, $contigs[$i]);

	next if($offset1 < $large_gap_size);
	$name1 =~ s/\{.*//; $name2 =~ s/\{.*//;
	next if($length{$name1} < 1000 || $length{$name2} < 1000);

	print "Considering gap: $name1 $or1 $name2 $or2 $offset1\n";
	$begin_gap_gc = ($or1 eq "BE" ? $end{$name1} : $begin{$name1});
	$end_gap_gc = ($or2 eq "EB" ? $end{$name2} : $begin{$name2});

	foreach $contig (@candidates) {

	    print "Fits: $contig BE $length{$contig} (GC info: $begin_gap_gc $begin{$contig} $end{$contig} $end_gap_gc)\n"
		if(abs($begin_gap_gc-$begin{$contig}) < $gc_diff &&
		   abs($end_gap_gc-$end{$contig}) < $gc_diff);
	    print "Fits: $contig EB $length{$contig} (GC info: $begin_gap_gc $end{$contig} $begin{$contig} $end_gap_gc)\n"
		if(abs($begin_gap_gc-$end{$contig}) < $gc_diff &&
		   abs($end_gap_gc-$begin{$contig}) < $gc_diff);
	}
	
	print "-"x80 . "\n";
    }
}

